@@ -12,26 +12,15 @@ import urllib.request
 from pathlib import Path, WindowsPath
 
 sys.path.insert(0, str(Path(__file__).parents[1]))
-# TODO? consider glob import or dotted access
-from pypdfium2_setup.packaging_base import (
-    Host,
-    SB_Dir,
-    DataTree,
-    PDFium_URL,
-    DepotTools_URL,
-    LibnameForSystem,
-    VerStatusFileName,
-    PlatformNames,
-    run_cmd,
-    call_ctypesgen,
-)
+# TODO consider dotted access?
+from pypdfium2_setup.packaging_base import *
 
 
 PatchDir       = SB_Dir / "patches"
 DepotToolsDir  = SB_Dir / "depot_tools"
 PDFiumDir      = SB_Dir / "pdfium"
 PDFiumBuildDir = PDFiumDir / "out" / "Default"
-OutputDir      = DataTree / PlatformNames.sourcebuild
+OutputDir      = DataTree / PlatNames.sourcebuild
 
 PatchesMain = [
     (PatchDir/"shared_library.patch", PDFiumDir),
@@ -45,7 +34,6 @@ PatchesWindows = [
 DefaultConfig = {
     "is_debug": False,
     "treat_warnings_as_errors": False,
-    # "clang_use_chrome_plugins": False,
     "pdf_is_standalone": True,
     "pdf_enable_v8": False,
     "pdf_enable_xfa": False,
@@ -60,6 +48,8 @@ SyslibsConfig = {
     "use_system_libpng": True,
     "use_system_zlib": True,
     "use_system_libtiff": True,
+    "clang_use_chrome_plugins": False,
+    "use_allocator_shim": False,
 }
 
 
@@ -146,8 +136,8 @@ def get_pdfium_version():
 
 
 def update_version(v_libpdfium):
-    ver_file = OutputDir / VerStatusFileName
-    ver_file.write_text( str(v_libpdfium) )
+    # NOTE this does not set the full version, not sure how to retrieve it
+    (OutputDir / VerStatusFileName).write_text( str(v_libpdfium) )
 
 
 def _create_resources_rc(v_libpdfium):
@@ -226,7 +216,7 @@ def pack(src_libpath, v_libpdfium, destname=None):
     update_version(v_libpdfium)
     
     include_dir = PDFiumDir / "public"
-    call_ctypesgen(OutputDir, include_dir)
+    call_ctypesgen(OutputDir, include_dir, pl_name=Host.platform)
 
 
 def get_tool(name):
@@ -238,20 +228,19 @@ def get_tool(name):
 
 def serialise_config(config_dict):
     
-    config_str = ""
-    sep = ""
+    parts = []
     
     for key, value in config_dict.items():
-        config_str += sep + f"{key} = "
+        p = f"{key} = "
         if isinstance(value, bool):
-            config_str += str(value).lower()
+            p += str(value).lower()
         elif isinstance(value, str):
-            config_str += f'"{value}"'
+            p += f'"{value}"'
         else:
             raise TypeError(f"Not sure how to serialise type {type(value).__name__}")
-        sep = "\n"
+        parts.append(p)
     
-    return config_str
+    return "\n".join(parts)
 
 
 def main(
