@@ -9,37 +9,27 @@ import tempfile
 from pathlib import Path
 
 sys.path.insert(0, str(Path(__file__).parents[1]))
-# TODO? consider glob import or dotted access
-from pypdfium2_setup.packaging_base import (
-    run_cmd,
-    clean_platfiles,
-    Host,
-    ModuleDir,
-    BindingsFileName,
-    LibnameForSystem,
-    BinaryPlatforms,
-    SourceTree,
-    BinaryTargetVar,
-    BinaryTarget_None,
-)
+# TODO consider dotted access?
+from pypdfium2_setup.packaging_base import *
 
 
-class ArtefactStash:
+class ArtifactStash:
     
     # Preserve in-tree aftefacts from editable install
     
     def __init__(self):
         
         self.tmpdir = None
-        file_names = [BindingsFileName, LibnameForSystem[Host.system]]
-        self.files = [fp for fp in [ModuleDir / fn for fn in file_names] if fp.exists()]
         
+        # FIXME some degree of duplication with base::get_platfiles()
+        file_names = [BindingsFileName, LibnameForSystem[Host.system]]
+        self.files = [fp for fp in [RawModuleDir / fn for fn in file_names] if fp.exists()]
         if len(self.files) == 0:
             return
         elif len(self.files) != 2:
             print(f"Warning: Expected exactly 2 platform files, but found {len(self.files)}.", file=sys.stderr)
         
-        self.tmpdir = tempfile.TemporaryDirectory(prefix="pypdfium2_artefact_stash_")
+        self.tmpdir = tempfile.TemporaryDirectory(prefix="pypdfium2_artifact_stash_")
         self.tmpdir_path = Path(self.tmpdir.name)
         for fp in self.files:
             shutil.move(fp, self.tmpdir_path)
@@ -48,7 +38,7 @@ class ArtefactStash:
         if self.tmpdir is None:
             return
         for fp in self.files:
-            shutil.move(self.tmpdir_path / fp.name, ModuleDir)
+            shutil.move(self.tmpdir_path / fp.name, RawModuleDir)
         self.tmpdir.cleanup()
 
 
@@ -59,18 +49,30 @@ def run_build(args):
 def main():
     
     parser = argparse.ArgumentParser(
-        description = "Craft sdist and wheels for pypdfium2, using `python3 -m build`. (This script does not take any arguments.)",
+        description = "Craft sdist and wheels for pypdfium2, using `python3 -m build`.",
     )
-    parser.parse_args()
+    parser.add_argument(
+        "--use-v8",
+        action = "store_true",
+    )
+    parser.add_argument(
+        "--version",
+        type = int,
+        default = None,
+    )
+    args = parser.parse_args()
+    if not args.version:
+        args.version = get_latest_version()
     
-    stash = ArtefactStash()
+    stash = ArtifactStash()
     
-    os.environ[BinaryTargetVar] = BinaryTarget_None
+    os.environ[PlatSpec_EnvVar] = PlatTarget_None
     run_build(["--sdist"])
     clean_platfiles()
     
+    suffix = (PlatSpec_V8Sym if args.use_v8 else "") + PlatSpec_VerSep + str(args.version)
     for plat in BinaryPlatforms:
-        os.environ[BinaryTargetVar] = plat
+        os.environ[PlatSpec_EnvVar] = plat + suffix
         run_build(["--wheel"])
         clean_platfiles()
     
