@@ -7,14 +7,8 @@ from pathlib import Path
 from wheel.bdist_wheel import bdist_wheel
 
 sys.path.insert(0, str(Path(__file__).parents[1]))
-# TODO? consider glob import or dotted access
-from pypdfium2_setup.packaging_base import (
-    VerNamespace,
-    LibnameForSystem,
-    plat_to_system,
-    get_wheel_tag,
-    emplace_platfiles,
-)
+# TODO consider dotted access?
+from pypdfium2_setup.packaging_base import *
 
 
 def bdist_factory(pl_name):
@@ -23,6 +17,7 @@ def bdist_factory(pl_name):
         
         def finalize_options(self, *args, **kws):
             bdist_wheel.finalize_options(self, *args, **kws)
+            # root_is_pure: redundant, covered by BinaryDistribution below - TODO consider removal
             self.root_is_pure = False
         
         def get_tag(self, *args, **kws):
@@ -31,26 +26,39 @@ def bdist_factory(pl_name):
     return pypdfium_bdist
 
 
+# Use a custom distclass declaring we have a binary extension, to prevent modules from being nested in a purelib/ subdirectory in wheels. This also sets `Root-Is-Purelib: false` in the WHEEL file.
 class BinaryDistribution (setuptools.Distribution):
     def has_ext_modules(self):
         return True
 
 
-SetupKws = dict(
-    version = VerNamespace["V_PYPDFIUM2"],
-)
+def get_setup_kws(modnames):
+    
+    # NOTE Handle python_requires dynamically so we could adjust it depending on which modules are included
+    
+    if set(modnames) == set(ModulesAll):
+        moddirs = {"": "src"}
+    else:
+        moddirs = {n: f"src/{n}" for n in [ModulesSpec_Dict[n] for n in modnames]}
+    
+    return dict(
+        version = VerNamespace["V_PYPDFIUM2"],
+        package_dir = moddirs,
+        python_requires = ">= 3.6",
+    )
 
 
-def mkwheel(pl_name):
+def mkwheel(pl_name, setup_kws):
+    
+    # NOTE This will not be called for helpers-only installs
     
     emplace_platfiles(pl_name)
-    
     system = plat_to_system(pl_name)
     libname = LibnameForSystem[system]
     
     setuptools.setup(
-        package_data = {"": [libname]},
+        package_data = {"pypdfium2_raw": [libname]},
         cmdclass = {"bdist_wheel": bdist_factory(pl_name)},
         distclass = BinaryDistribution,
-        **SetupKws,
+        **setup_kws,
     )
