@@ -4,30 +4,17 @@
 
 import sys
 import shutil
-import tarfile
 import argparse
 import traceback
 import functools
-import os.path
 from pathlib import Path
 from urllib import request
 from concurrent.futures import ThreadPoolExecutor
 
 sys.path.insert(0, str(Path(__file__).parents[1]))
-# TODO? consider glob import or dotted access
-from pypdfium2_setup.packaging_base import (
-    Host,
-    DataTree,
-    VerStatusFileName,
-    V8StatusFileName,
-    ReleaseNames,
-    BinaryPlatforms,
-    ReleaseURL,
-    BinaryTarget_Auto,
-    get_latest_version,
-    call_ctypesgen,
-    emplace_platfiles,
-)
+from pypdfium2_setup._compat import safer_tar_unpack
+# TODO consider dotted access?
+from pypdfium2_setup.packaging_base import *
 
 
 def clear_data(download_files):
@@ -60,13 +47,10 @@ def _get_package(pl_name, version, robust, use_v8):
         else:
             raise
     
-    if use_v8:
-        (pl_dir / V8StatusFileName).touch(exist_ok=True)
-    
     return pl_name, fp
 
 
-def download_releases(version, platforms, robust, max_workers, use_v8):
+def download_releases(platforms, version, use_v8, max_workers, robust):
     
     if not max_workers:
         max_workers = len(platforms)
@@ -81,104 +65,83 @@ def download_releases(version, platforms, robust, max_workers, use_v8):
     return archives
 
 
-def safe_extract(tar, dest_dir, **kwargs):
-    
-    # Workaround against CVE-2007-4559 (path traversal attack) (thanks @Kasimir123 / @TrellixVulnTeam)
-    # Actually, we would just like to use shutil.unpack_archive() or similar, but to the author's knowledge, the stdlib still does not provide any (simple) means to extract tars safely (as of Feb 2023).
-    # It is not understandable why they don't just add an option `prevent_traversal` or something to shutil.unpack_archive().
-    
-    dest_dir = dest_dir.resolve()
-    for member in tar.getmembers():
-        # if not (dest_dir/member.name).resolve().is_relative_to(dest_dir):  # python >= 3.9
-        if str(dest_dir) != os.path.commonpath( [dest_dir, (dest_dir/member.name).resolve()] ):
-            raise RuntimeError("Attempted path traversal in tar archive (probably malicious).")
-    tar.extractall(dest_dir, **kwargs)
-
+# TODO Do not unpack whole archives, instead extract only the binaries we need and retrieve headers from pdfium directly. This would allow us to get rid of the tar compat code.
 
 def unpack_archives(archives):
-    for pl_name, fp in archives.items():
+    for pl_name, archive_path in archives.items():
         dest_dir = DataTree / pl_name / "build_tar"
-        with tarfile.open(fp) as archive:
-            safe_extract(archive, dest_dir)
-        fp.unlink()
+        safer_tar_unpack(archive_path, dest_dir)
+        archive_path.unlink()
 
 
-def generate_bindings(archives, version):
+def generate_bindings(archives, version, full_version, use_v8, ctypesgen_kws):
     
     for pl_name in archives.keys():
         
         pl_dir = DataTree / pl_name
         build_dir = pl_dir / "build_tar"
         bin_dir = build_dir / "lib"
-        dirname = pl_dir.name
         
-        if dirname.startswith("windows"):
-            target_name = "pdfium.dll"
+        system = plat_to_system(pl_name)
+        if system == SysNames.windows:
             bin_dir = build_dir / "bin"
-        elif dirname.startswith("darwin"):
-            target_name = "pdfium.dylib"
-        elif "linux" in dirname:
-            target_name = "pdfium"
-        else:
-            raise ValueError(f"Unknown platform directory name '{dirname}'")
         
-        items = list(bin_dir.iterdir())
-        assert len(items) == 1
-        shutil.move(bin_dir/items[0], pl_dir/target_name)
+        libname = LibnameForSystem[system]
+        src_libpath = bin_dir / libname
+        assert src_libpath.is_file()
+        shutil.copyfile(src_libpath, pl_dir/libname)
         
         ver_file = DataTree / pl_name / VerStatusFileName
-        ver_file.write_text(str(version))
+        ver_file.write_text(f"{version}\n{full_version}")
+        v8_file = (pl_dir / V8StatusFileName)
+        if use_v8:
+            v8_file.touch(exist_ok=True)
+        else:
+            assert not v8_file.exists()
         
-        call_ctypesgen(pl_dir, build_dir/"include")
+        call_ctypesgen(pl_dir, build_dir/"include", pl_name=pl_name, use_v8xfa=use_v8, **ctypesgen_kws)
         shutil.rmtree(build_dir)
 
 
-def main(platforms, version=None, robust=False, max_workers=None, use_v8=False, emplace=False):
-    
-    # FIXME questionable code style?
+def main(platforms, version=None, robust=False, max_workers=None, use_v8=False, ctypesgen_kws={}):
     
     if not version:
         version = get_latest_version()
-    
     if not platforms:
-        platforms = [Host.platform] if emplace else BinaryPlatforms
-    
+        platforms = BinaryPlatforms
     if len(platforms) != len(set(platforms)):
         raise ValueError("Duplicate platforms not allowed.")
-    if BinaryTarget_Auto in platforms:
-        platforms = platforms.copy()
-        platforms[platforms.index(BinaryTarget_Auto)] = Host.platform
-
-    clear_data(platforms)
-    archives = download_releases(version, platforms, robust, max_workers, use_v8)
-    unpack_archives(archives)
-    generate_bindings(archives, version)
     
-    if emplace:
-        emplace_platfiles(Host.platform)
+    full_version = get_full_version(version)
+    clear_data(platforms)
+    archives = download_releases(platforms, version, use_v8, max_workers, robust)
+    unpack_archives(archives)
+    generate_bindings(archives, version, full_version, use_v8, ctypesgen_kws)
 
 
+# low-level CLI interface for testing - users should go with higher-level emplace.py or setup.py
+
 def parse_args(argv):
-    platform_choices = (BinaryTarget_Auto, *BinaryPlatforms)
     parser = argparse.ArgumentParser(
         description = "Download pre-built PDFium packages and generate bindings.",
     )
     parser.add_argument(
+        # FIXME with metavar, choices are not visible in help by default - without it, the long choices list is repeated 4 times due to 2 flags and nargs="+"
         "--platforms", "-p",
         nargs = "+",
-        metavar = "identifier",
-        choices = platform_choices,
-        help = f"The platform(s) to include. `auto` represents the current host platform. Choices: {platform_choices}.",
+        metavar = "ID",
+        choices = BinaryPlatforms,
+        help = f"The platform(s) to include. Choices: {BinaryPlatforms}",
+    )
+    parser.add_argument(
+        "--use-v8",
+        action = "store_true",
+        help = "Use V8 binaries (JavaScript/XFA support)."
     )
     parser.add_argument(
         "--version", "-v",
         type = int,
-        help = "The pdfium-binaries release to use (defaults to latest). Must be a valid tag integer."
-    )
-    parser.add_argument(
-        "--robust",
-        action = "store_true",
-        help = "Skip missing binaries instead of raising an exception.",
+        help = "The binaries release to use (defaults to latest). Must be a valid tag integer."
     )
     parser.add_argument(
         "--max-workers",
@@ -186,14 +149,9 @@ def parse_args(argv):
         help = "Maximum number of jobs to run in parallel when downloading binaries.",
     )
     parser.add_argument(
-        "--use-v8",
+        "--robust",
         action = "store_true",
-        help = "Use PDFium V8 binaries for JavaScript and XFA support."
-    )
-    parser.add_argument(
-        "--emplace",
-        action = "store_true",
-        help = "Move binaries for the host platform into the source tree.",
+        help = "Skip missing binaries instead of raising an exception.",
     )
     return parser.parse_args(argv)
 
