@@ -8,8 +8,8 @@ import logging
 import weakref
 from collections import namedtuple
 import pypdfium2.raw as pdfium_c
-from pypdfium2._helpers.misc import PdfiumError
 import pypdfium2.internal as pdfium_i
+from pypdfium2._helpers.misc import PdfiumError
 
 logger = logging.getLogger(__name__)
 
@@ -165,9 +165,17 @@ class PdfBitmap (pdfium_i.AutoCloseable):
         pdfium_c.FPDFBitmap_FillRect(self, left, top, width, height, c_color)
     
     
-    # Requirement: If the result is a view of the buffer (not a copy), it keeps the buffer's memory valid by retaining a reference to the buffer object.
-    # As of May 2023, this seems to hold true for numpy and PIL.
-    # TODO? Consider attaching a buffer keep-alive finalizer to the converted object
+    # Requirement: If the result is a view of the buffer (not a copy), it keeps the referenced memory valid.
+    # 
+    # Note that memory management differs between native and foreign bitmap buffers:
+    # - With native bitmaps, the memory is allocated by python on creation of the buffer object (transparent).
+    # - With foreign bitmaps, the buffer object is merely a view of memory allocated by pdfium and will be freed by finalizer (opaque).
+    # 
+    # It is necessary that receivers correctly handle both cases, e.g. by keeping the buffer object itself alive.
+    # As of May 2023, this seems to hold true for NumPy and PIL. New converters should be carefully tested.
+    # 
+    # We could consider attaching a buffer keep-alive finalizer to any converted objects referencing the buffer,
+    # but then we'd have to rely on third parties to actually create a reference at all times, otherwise we would unnecessarily delay releasing memory.
     
     
     def to_numpy(self):
@@ -204,12 +212,13 @@ class PdfBitmap (pdfium_i.AutoCloseable):
         Convert the bitmap to a :mod:`PIL` image, using :func:`PIL.Image.frombuffer`.
         
         For ``RGBA``, ``RGBX`` and ``L`` buffers, PIL is supposed to share memory with
-        the original bitmap buffer, so changes to the buffer should be reflected in the image
-        (however, changes to the image may not be reflected in the buffer due to PIL's behaviour).
+        the original bitmap buffer, so changes to the buffer should be reflected in the image, and vice versa.
         Otherwise, PIL will make a copy of the data.
         
         Returns:
             PIL.Image.Image: PIL image (representation or copy of the bitmap buffer).
+        
+        .. versionchanged:: 4.16 Set ``image.readonly = False`` so that changes to the image are also reflected in the buffer.
         """
         
         # https://pillow.readthedocs.io/en/stable/reference/Image.html#PIL.Image.frombuffer
@@ -225,22 +234,31 @@ class PdfBitmap (pdfium_i.AutoCloseable):
             self.stride,                # bytes per line
             1,                          # orientation (top->bottom)
         )
+        image.readonly = False
         
         return image
     
     
+    # FIXME might want to rename *recopy* to *mutable* ?
     @classmethod
-    def from_pil(cls, pil_image):
+    def from_pil(cls, pil_image, recopy=False):
         """
         Convert a :mod:`PIL` image to a PDFium bitmap.
         Due to the restricted number of color formats and bit depths supported by PDFium's
         bitmap implementation, this may be a lossy operation.
         
-        Note that this uses the memory segment of an immutable bytes object as buffer to avoid an additional layer of copying.
-        Therefore, modifying the buffer of a bitmap created with this method (e.g. using :meth:`.fill_rect`) is against Python's API contract.
-        
+        Parameters:
+            pil_image (PIL.Image.Image):
+                The image.
+            recopy (bool):
+                If False (the default), reuse the memory segment of an immutable bytes object as buffer to avoid an additional layer of copying. This is recommended if you do not modify the bitmap, though the buffer does not actually enforce immutability.
+                If True (otherwise), copy memory into a new buffer that is mutable by design. This is recommended if you modify the bitmap, e.g. using :meth:`.fill_rect`.
+                Note that the resulting bitmap is always independent of the PIL image, regardless of this option.
         Returns:
             PdfBitmap: PDFium bitmap (with a copy of the PIL image's data).
+        
+        .. versionchanged:: 4.15 reference bytes object instead of copying
+        .. versionadded:: 4.16 opt-in re-copying for mutability within Python API contract
         """
         
         if pil_image.mode in pdfium_i.BitmapStrToConst:
@@ -250,13 +268,16 @@ class PdfBitmap (pdfium_i.AutoCloseable):
             pil_image = _pil_convert_for_pdfium(pil_image)
             format = pdfium_i.BitmapStrReverseToConst[pil_image.mode]
         
-        # see above and https://stackoverflow.com/a/21490290/15547292
         py_buffer = pil_image.tobytes()
-        c_buffer = ctypes.cast(py_buffer, ctypes.POINTER(ctypes.c_ubyte * len(py_buffer))).contents
-        weakref.finalize(c_buffer, lambda: id(py_buffer))  # hack to keep the underlying bytes object alive
-        width, height = pil_image.size
+        if recopy:
+            c_buffer = (ctypes.c_ubyte * len(py_buffer)).from_buffer_copy(py_buffer)
+        else:
+            # see docs above and https://stackoverflow.com/a/21490290/15547292
+            c_buffer = ctypes.cast(py_buffer, ctypes.POINTER(ctypes.c_ubyte * len(py_buffer))).contents
+            weakref.finalize(c_buffer, lambda: id(py_buffer))
         
-        return cls.new_native(width, height, format, rev_byteorder=False, buffer=c_buffer)
+        w, h = pil_image.size
+        return cls.new_native(w, h, format, rev_byteorder=False, buffer=c_buffer)
     
     
     # TODO implement from_numpy()
@@ -264,6 +285,8 @@ class PdfBitmap (pdfium_i.AutoCloseable):
 
 def _pil_convert_for_pdfium(pil_image):
     
+    # FIXME? convoluted / hard to understand; improve control flow
+    
     if pil_image.mode == "1":
         pil_image = pil_image.convert("L")
     elif pil_image.mode.startswith("RGB"):
@@ -281,9 +304,9 @@ def _pil_convert_for_pdfium(pil_image):
         r, g, b, a = pil_image.split()
         pil_image = PIL.Image.merge("RGBA", (b, g, r, a))
     elif pil_image.mode == "RGBX":
-        # no one needs the unused channel, so drop it to reduce memory consumption
-        r, g, b, _ = pil_image.split()
-        pil_image = PIL.Image.merge("RGB", (b, g, r))
+        # technically the x channel may be unnecessary, but preserve what the caller passes in
+        r, g, b, x = pil_image.split()
+        pil_image = PIL.Image.merge("RGBX", (b, g, r, x))
     
     return pil_image
 
