@@ -1,27 +1,25 @@
 # SPDX-FileCopyrightText: 2023 geisserml <geisserml@gmail.com>
 # SPDX-License-Identifier: Apache-2.0 OR BSD-3-Clause
 
-__all__ = ["PdfObject", "PdfImage"]
+__all__ = ("PdfObject", "PdfImage")
 
 import ctypes
+from ctypes import c_uint, c_float
 from pathlib import Path
 from collections import namedtuple
 import pypdfium2.raw as pdfium_c
+import pypdfium2.internal as pdfium_i
 from pypdfium2._helpers.misc import PdfiumError
 from pypdfium2._helpers.matrix import PdfMatrix
 from pypdfium2._helpers.bitmap import PdfBitmap
-from pypdfium2._helpers._internal import consts, utils
-from pypdfium2._helpers._internal.bases import AutoCloseable
 
 try:
     import PIL.Image
 except ImportError:
     PIL = None
 
-c_float = ctypes.c_float
 
-
-class PdfObject (AutoCloseable):
+class PdfObject (pdfium_i.AutoCloseable):
     """
     Page object helper class.
     
@@ -58,10 +56,7 @@ class PdfObject (AutoCloseable):
     
     def __init__(self, raw, page=None, pdf=None, level=0):
         
-        self.raw = raw
-        self.page = page
-        self.pdf = pdf
-        self.level = level
+        self.raw, self.page, self.pdf, self.level = raw, page, pdf, level
         
         if page is not None:
             if self.pdf is None:
@@ -69,15 +64,13 @@ class PdfObject (AutoCloseable):
             elif self.pdf is not page.pdf:
                 raise ValueError("*page* must belong to *pdf* when constructing a pageobject.")
         
-        AutoCloseable.__init__(self, pdfium_c.FPDFPageObj_Destroy, needs_free=(page is None))
+        super().__init__(pdfium_c.FPDFPageObj_Destroy, needs_free=(page is None))
     
     
     @property
     def parent(self):  # AutoCloseable hook
-        if self.page is not None:
-            return self.page
-        else:
-            return self.pdf  # may be none
+        # May be None (loose pageobject)
+        return self.pdf if self.page is None else self.page
     
     
     def get_pos(self):
@@ -90,12 +83,12 @@ class PdfObject (AutoCloseable):
         if self.page is None:
             raise RuntimeError("Must not call get_pos() on a loose pageobject.")
         
-        left, bottom, right, top = c_float(), c_float(), c_float(), c_float()
-        success = pdfium_c.FPDFPageObj_GetBounds(self, left, bottom, right, top)
-        if not success:
+        l, b, r, t = c_float(), c_float(), c_float(), c_float()
+        ok = pdfium_c.FPDFPageObj_GetBounds(self, l, b, r, t)
+        if not ok:
             raise PdfiumError("Failed to locate pageobject.")
         
-        return (left.value, bottom.value, right.value, top.value)
+        return (l.value, b.value, r.value, t.value)
     
     
     def get_matrix(self):
@@ -104,8 +97,8 @@ class PdfObject (AutoCloseable):
             PdfMatrix: The pageobject's current transform matrix.
         """
         fs_matrix = pdfium_c.FS_MATRIX()
-        success = pdfium_c.FPDFPageObj_GetMatrix(self, fs_matrix)
-        if not success:
+        ok = pdfium_c.FPDFPageObj_GetMatrix(self, fs_matrix)
+        if not ok:
             raise PdfiumError("Failed to get matrix of pageobject.")
         return PdfMatrix.from_raw(fs_matrix)
     
@@ -115,8 +108,8 @@ class PdfObject (AutoCloseable):
         Parameters:
             matrix (PdfMatrix): Set this matrix as the pageobject's transform matrix.
         """
-        success = pdfium_c.FPDFPageObj_SetMatrix(self, matrix)
-        if not success:
+        ok = pdfium_c.FPDFPageObj_SetMatrix(self, matrix)
+        if not ok:
             raise PdfiumError("Failed to set matrix of pageobject.")
     
     
@@ -135,6 +128,7 @@ class PdfImage (PdfObject):
     Image object helper class (specific kind of page object).
     """
     
+    # cf. https://crbug.com/pdfium/1203
     #: Filters applied by :func:`FPDFImageObj_GetImageDataDecoded`. Hereafter referred to as "simple filters", while non-simple filters will be called "complex filters".
     SIMPLE_FILTERS = ("ASCIIHexDecode", "ASCII85Decode", "RunLengthDecode", "FlateDecode", "LZWDecode")
     
@@ -150,36 +144,45 @@ class PdfImage (PdfObject):
             This means that new images will appear as a tiny square of 1x1 units on the bottom left corner of the page.
             Use :class:`.PdfMatrix` and :meth:`.set_matrix` to adjust size and position.
         """
-        return cls(
-            pdfium_c.FPDFPageObj_NewImageObj(pdf),
-            page = None,
-            pdf = pdf,
-        )
+        raw_img = pdfium_c.FPDFPageObj_NewImageObj(pdf)
+        return cls(raw_img, page=None, pdf=pdf)
     
     
     def get_metadata(self):
         """
-        Retrieve image metadata, including dimensions, DPI, bits per pixel, and color space.
+        Retrieve image metadata including DPI, bits per pixel, color space, and size.
         If the image does not belong to a page yet, bits per pixel and color space will be unset (0).
         
         Note:
             * The DPI values signify the resolution of the image on the PDF page, not the DPI metadata embedded in the image file.
-            * Due to issues in PDFium, this function can be inefficient.
+            * Due to issues in PDFium, this function can be slow. If you only need image size, prefer the faster :meth:`.get_size` instead.
         
         Returns:
             FPDF_IMAGEOBJ_METADATA: Image metadata structure
         """
-        
         # https://crbug.com/pdfium/1928
-        
         metadata = pdfium_c.FPDF_IMAGEOBJ_METADATA()
-        success = pdfium_c.FPDFImageObj_GetImageMetadata(self, self.page, metadata)
-        if not success:
+        ok = pdfium_c.FPDFImageObj_GetImageMetadata(self, self.page, metadata)
+        if not ok:
             raise PdfiumError("Failed to get image metadata.")
-        
         return metadata
     
     
+    def get_size(self):
+        """
+        .. versionadded:: 4.8/5731
+        
+        Returns:
+            (int, int): Image dimensions as a tuple of (width, height).
+        """
+        # https://pdfium-review.googlesource.com/c/pdfium/+/106290
+        w, h = c_uint(), c_uint()
+        ok = pdfium_c.FPDFImageObj_GetImagePixelSize(self, w, h)
+        if not ok:
+            raise PdfiumError("Failed to get image size.")
+        return w.value, h.value
+    
+    
     def load_jpeg(self, source, pages=None, inline=False, autoclose=True):
         """
         Set a JPEG as the image object's content.
@@ -201,30 +204,26 @@ class PdfImage (PdfObject):
         if isinstance(source, (str, Path)):
             buffer = open(source, "rb")
             autoclose = True
-        elif utils.is_buffer(source, "r"):
+        elif pdfium_i.is_buffer(source, "r"):
             buffer = source
         else:
             raise ValueError(f"Cannot load JPEG from {source} - not a file path or byte buffer.")
         
-        bufaccess, ld_data = utils.get_bufreader(buffer)
+        bufaccess, to_hold = pdfium_i.get_bufreader(buffer)
+        loader = pdfium_c.FPDFImageObj_LoadJpegFileInline if inline else pdfium_c.FPDFImageObj_LoadJpegFile
         
-        if inline:
-            loader = pdfium_c.FPDFImageObj_LoadJpegFileInline
-        else:
-            loader = pdfium_c.FPDFImageObj_LoadJpegFile
-        
-        c_pages, page_count = utils.pages_c_array(pages)
-        success = loader(c_pages, page_count, self, bufaccess)
-        if not success:
+        c_pages, page_count = pdfium_i.pages_c_array(pages)
+        ok = loader(c_pages, page_count, self, bufaccess)
+        if not ok:
             raise PdfiumError("Failed to load JPEG into image object.")
         
         if inline:
-            for data in ld_data:
+            for data in to_hold:
                 id(data)
             if autoclose:
                 buffer.close()
         else:
-            self.pdf._data_holder += ld_data
+            self.pdf._data_holder += to_hold
             if autoclose:
                 self.pdf._data_closer.append(buffer)
     
@@ -240,9 +239,9 @@ class PdfImage (PdfObject):
             pages (list[PdfPage] | None):
                 A list of loaded pages that might contain the image object. See :meth:`.load_jpeg`.
         """
-        c_pages, page_count = utils.pages_c_array(pages)
-        success = pdfium_c.FPDFImageObj_SetBitmap(c_pages, page_count, self, bitmap)
-        if not success:
+        c_pages, page_count = pdfium_i.pages_c_array(pages)
+        ok = pdfium_c.FPDFImageObj_SetBitmap(c_pages, page_count, self, bitmap)
+        if not ok:
             raise PdfiumError("Failed to set image to bitmap.")
     
     
@@ -279,17 +278,10 @@ class PdfImage (PdfObject):
         Returns:
             ctypes.Array: The data of the image stream (as :class:`~ctypes.c_ubyte` array).
         """
-        
-        if decode_simple:
-            # apply simple filters (see https://crbug.com/pdfium/1203 for description)
-            func = pdfium_c.FPDFImageObj_GetImageDataDecoded
-        else:
-            func = pdfium_c.FPDFImageObj_GetImageDataRaw
-        
+        func = pdfium_c.FPDFImageObj_GetImageDataDecoded if decode_simple else pdfium_c.FPDFImageObj_GetImageDataRaw
         n_bytes = func(self, None, 0)
         buffer = (ctypes.c_ubyte * n_bytes)()
         func(self, buffer, n_bytes)
-        
         return buffer
     
     
@@ -342,10 +334,12 @@ class PdfImage (PdfObject):
         extraction_gen = _extract_smart(self, *args, **kwargs)
         format = next(extraction_gen)
         
-        if isinstance(dest, (str, Path)):
-            with open(f"{dest}.{format}", "wb") as buf:
+        if isinstance(dest, str):
+            dest = Path(dest)
+        if isinstance(dest, Path):
+            with open(dest.with_suffix("."+format), "wb") as buf:
                 extraction_gen.send(buf)
-        elif utils.is_buffer(dest, "w"):
+        elif pdfium_i.is_buffer(dest, "w"):
             extraction_gen.send(dest)
         else:
             raise ValueError(f"Cannot extract to '{dest}'")
@@ -375,15 +369,15 @@ def _get_pil_mode(colorspace, bpp):
 
 def _extract_smart(image_obj, fb_format=None, fb_render=False):
     
-    pil_image = None
-    data = None
-    info = None
+    # FIXME somewhat hard to read...
     
     try:
         data, info = _extract_direct(image_obj)
     except ImageNotExtractableError:
+        # TODO? log reason why the image cannot be extracted directly
         pil_image = image_obj.get_bitmap(render=fb_render).to_pil()
     else:
+        pil_image = None
         format = info.format
         if format == "raw":
             metadata = info.metadata
@@ -395,23 +389,12 @@ def _extract_smart(image_obj, fb_format=None, fb_render=False):
             )
     
     if pil_image:
-        if fb_format:
-            format = fb_format
-        elif pil_image.mode == "CMYK":
-            format = "tiff"
-        else:
-            format = "png"
+        format = fb_format if fb_format else "tiff" if pil_image.mode == "CMYK" else "png"
     
-    # provide format, receive buffer
     buffer = yield format
+    pil_image.save(buffer, format=format) if pil_image else buffer.write(data)
     
-    if pil_image:
-        pil_image.save(buffer, format=format)
-    else:
-        buffer.write(data)
-    
-    # breakpoint preventing StopIteration on send()
-    yield None
+    yield  # breakpoint preventing StopIteration on .send()
 
 
 def _extract_direct(image_obj):
@@ -421,18 +404,12 @@ def _extract_direct(image_obj):
     metadata = image_obj.get_metadata()
     mode = _get_pil_mode(metadata.colorspace, metadata.bits_per_pixel)
     
-    out_data = None
-    out_format = None
-    
-    # Not sure if FlateDecode or LZWDecode data could be wrapped directly in an image file structure like PNG or TIFF
-    
     if len(complex_filters) == 0:
         if mode:
             out_data = image_obj.get_data(decode_simple=True)
             out_format = "raw"
         else:
-            raise ImageNotExtractableError(f"Unhandled color space {consts.ColorspaceToStr.get(metadata.colorspace)} - don't know how to treat data")
-    
+            raise ImageNotExtractableError(f"Unhandled color space {pdfium_i.ColorspaceToStr.get(metadata.colorspace)} - don't know how to treat data.")
     elif len(complex_filters) == 1:
         f = complex_filters[0]
         if f == "DCTDecode":
@@ -442,14 +419,9 @@ def _extract_direct(image_obj):
             out_data = image_obj.get_data(decode_simple=True)
             out_format = "jp2"
         else:
-            raise ImageNotExtractableError(f"Unhandled complex filter {f}")
-        
-        # Other complex filters:
-        # CCITTFaxDecode: In theory, could be extracted directly (with a TIFF header builder like pikepdf/models/_transcoding.py:generate_ccitt_header), but PDFium doesn't tell us which CCITT group encoding it is.
-        # JBIG2Decode: In PDF, JBIG2 header info is stripped, and global segments may be stored in a separate stream. In that form, the data would probably not be of much use, except perhaps for direct re-insertion into another PDF. We're not sure if it would be possible to re-combine this into a single JBIG2 file, or if any application could use this at all. PDFium doesn't provide us with the global segments, anyway.
-    
+            raise ImageNotExtractableError(f"Unhandled complex filter {f}.")
     else:
-        raise ImageNotExtractableError(f"Cannot handle multiple complex filters {complex_filters}")
+        raise ImageNotExtractableError(f"Cannot handle multiple complex filters {complex_filters}.")
     
     info = ImageInfo(out_format, mode, metadata, all_filters, complex_filters)
     return out_data, info
